/*
META-ANALYSIS DEMONSTRATION PROGRAM (L. MADDEN AND P. PAUL [OHIO STATE UNIVERSITY]).

Program designed for univariate fixed- and random-effects meta-analysis. 
This version is for SAS 9.3.

Below, a data set supplied by Shah and Dillard (see Plant Disease 90: 1413-1418 [2006])
is analyzed using metaa-analysis. 

First, several macros (written by L. Madden, or kindly made available by D. Wilson), are listed.
These are intended to streamline the analysis and the presentation of the data in graphic form.
One could also put these macros in a separate sas file, and simply call them from this program 
using an %include statement.

Highlight all the macros and run. Then you can just scroll past these macros to see 
the data set and start the analysis (by running portions of this program file). 

Most of the calculations are explained in Madden & Paul (Phytopathology 102:16-30 [2011]), 
and references given in that publications.
*/

run;
%MACRO metareg(es,w,x,dsn=_last_,model=fe) ;
*---Revised Lipsey-Wilson macro, 7/2005 (L. Madden). This version CORRECTLY does the ML estimation
	(but maybe not REML). Macro made available for distribution courtesy of David Wilson.
	If you use, please cite book: Practical Meta-Analyis, by M. W. Lipsey & D. B. Wilson. 
	2001. Sage Publications, Thousand Oaks, USA.;
*---Does moment (MM) or ML estimation.;

      proc iml;

        use &dsn ;
        read all var {&es} into es  where(&es^=. & &w^=. & &x^=.) ;
        read all var {&w}  into w   where(&es^=. & &w^=. & &x^=.) ;
        read all var {&x}  into x   where(&es^=. & &w^=. & &x^=.) ;

        k = nrow(es)  ;
        df = k - 1   ;
        j = j(k,1,1) ;
        x = j(k,1,1) || x ;
        p = j(1,ncol(x),1) ;
		v = 1/w;

        %if %upcase(&model) = MM | %upcase(&model) = ML %then
           %do;
                  xw = x#(w*p) ; xwx = T(xw)*x ;
                  B = inv(xwx)*T(xw)*es ;
                  qe = sum(es#w#es) - T(B)*xwx*B ;
                  ww = w#w ; xww = x#(ww*p) ; xwwx = T(xww)*x ;
                  c = (qe-(k-ncol(x)))*inv(sum(w) - trace((xwwx)*inv(xwx))) ;
                  if c<0 then c = 0 ;
                  w = 1/(1/w + c) ;
            %end ;

        %if %upcase(&model) = ML %then
           %do i=1 %to 100;
                          xw = x#(w*p) ; xwx = T(xw)*x ;
                          B = inv(xwx)*T(xw)*es ;
                          r = es - x*B ;
                          c = sum((w#w)#(r#r-(v)))/
                                (sum(w#w)) ;
                          if c<0 then c = 0 ;
                          w = 1/(v+c) ;

			 %end ;

		%if %upcase(&model) = ML %then
			%do ;  se_c = sqrt(2/sum(w#w)) ; %end ;

        xw = x#(w*p) ; xwx = T(xw)*x ;
        B = inv(xwx)*T(xw)*es ;

        meanes = sum(es#w)/sum(w) ;
        q =  sum((meanes - es)#(meanes - es)#w) ;
        qe = sum(es#w#es) - T(B)*xwx*B ;
        qr = q - qe ;
        dfe = nrow(es)-ncol(x) ;
        dfr = ncol(x)-1 ;
        dft = nrow(es)-1 ;
        pt = 1 - probchi(q,dft)  ;
        pe = 1 - probchi(qe,dfe) ;
        pr = 1 - probchi(qr,dfr) ;
        se = sqrt(vecdiag(inv(xwx))) ;
        zvalues = B#vecdiag(inv(diag(se))) ;
        pvalues = (1 - probnorm(abs(zvalues)))*2 ;
        lowerB = B - se*1.96 ;
        upperB = B + se*1.96 ;

        d = x - T(T((x#(w*p))[+,])*T(j)/sum(w)) ;
        sx = sqrt(vecdiag(T(d#(w*p))*d/sum(w))) ;
          sy = sqrt(q/sum(w)) ;
        beta = (B#sx) *inv(sy) ;
        r2 = qr/(qr+qe) ;
              %if %upcase(&model) = MM %then
                %do ; print "--------- Random Effects Model ------" ;
                      print "-- estimated via method of moments --" ;
                print c [label= ' '] [rowname = 'Random eff. var. component = ']
                                [format=12.8];
                %end ;

              %if %upcase(&model) = ML %then
                %do ; print "--------- Random Effects Model ------" ;
                      print "-- estimated via iterative full information maximum likelihood --" ;
                mlc = c // se_c ;
				print mlc [label= ' '] [rowname = {'Random eff. var. component  = '
				                                  'St. Error of var. component = '}]
                                [format=12.8];
                %end ;


              print "------- Descriptives -------" ;
              print k      [label="No. of Obs. "]
                         meanes [label="Mean Obs."] [format=10.4]
                        r2     [label="R-squared"] [format=10.4] ;

              print  "------- Homogeneity Analysis  -------";
                qtab1 = qr // qe // q ;
                qtab2 = dfr // dfe // dft ;
                qtab3 = pr // pe // pt ;
                print qtab1 [rowname={"Model","Residual","Total"}]
                            [colname="Q"][label="Source"] [format=12.5]
                            qtab2 [label="df"]
                            qtab3 [label="p"] [format=12.5];

              print "------- Regression Coefficients -------" ;
              names = {&x} ;
              names = T(names) ;
              group = {"Constant"} // names ;
              print b [rowname=group] [colname="B"] [label="Variable"]
                          [format=10.5]
                        se [label="SE"] [format=10.5]
                        lowerB [label='-95%CI'] [format=10.5]
                        upperB [label='+95%CI'] [format=10.5]
                        zvalues  [label="z"] [format=10.5]
                        pvalues  [label="p"] [format=10.5]
                        beta     [label="beta"] [format=10.5] ;

%MEND metareg ;
run; quit;


%MACRO meanes(es,w,dsn=_last_,print=raw) ;	
*---Lipsey-Wilson macro for mean effect size and other statistics,
	based on MOMENT-type calculations. Macro made available for distribution courtesy of 
	David Wilson. If you use, please cite book: Practical Meta-Analyis, by M. W. Lipsey 
	& D. B. Wilson. 2001. Sage Publications, Thousand Oaks, USA.
	[Some macro additions by L. Madden];
        proc iml;
                use &dsn ;
                read all var{&es} into es   where(&es^=. & &w^=.) ;
                read all var{&w}  into w    where(&es^=. & &w^=.) ;

                k = nrow(es)  ;
                df = k - 1   ;

                mes = sum(es#w)/sum(w);
                sem = sqrt(1/sum(w));
                les = mes - 1.95996*sem;
                ues = mes + 1.95996*sem;
                z   = mes/sem ;
                pz  = (1 - (.5+erf(abs(z)/sqrt(2))/2))*2 ;
                maxes = max(es) ;
                mines = min(es) ;

                q = sum((es#es)#w) - sum(es#w)#sum(es#w)/sum(w);
                wsd   = sqrt(q*w[+,]**-1) ;
                pq   = 1-probchi(q,df) ;

				H2 = q/df;	*---Hedges Hsquared statistic;
				I2 = 100*(H2-1)/H2; if (I2 < 0) then I2 = 0;  *---Hedges Isquared statistic;

                c    = (q - df)/(w[+,]-sum(w#w)/w[+,]) ;
                if c<0 then ; do ; c = 0 ; end ;
                wre    = 1/(1/w + c) ;

                mesre = sum(es#wre)/sum(wre) ;
                semre = sqrt(1/sum(wre)) ;
                lesre = mesre -    1.95996*semre ;
                uesre = mesre +    1.95996*semre ;
                zre   = mesre/semre ;
                pzre  = (1 - (.5+erf(abs(zre)/sqrt(2))/2))*2 ;

				R2 = (semre/sem)**2;
                %if %upcase(&print) = EXP %then
                        %do;
                                mes = exp(mes) ;
                                les = exp(les) ;
                                ues = exp(ues) ;
                                sem = . ;
                                mes_re = exp(mes_re) ;
                                les_re = exp(les_re) ;
                                ues_re = exp(ues_re) ;
                                semre = . ;
                %end;

                print '-------------- Distribution Description --------------';
                mattrib         k     label={"No. of obs."}     maxes label={"Max Obs."}
                        mines label={"Min Obs."} wsd   label={"Weighted SD"};
                print k mines maxes wsd [format=12.5];
                print " ";
                print '-------------- Homogeneity Analysis --------------';
                mattrib df label={"df"} pq label={"p"} ;
                print q [format=12.5] df pq [format=12.5];
                print " ";
				
				print " ";

                print '-------------- Fixed & Random Effects Model --------------';
                fixed = mes || sem || les || ues || z || pz ;
                random = mesre || semre || lesre || uesre || zre || pzre ;

                model = fixed // random ;
                mattrib model rowname=({'Fixed', 'Random'})
                        colname=({'Mean' 'SE' '-95%CI' '+95%CI' 'z' 'p'})
                        label={"Model"};
                print model [format=10.5];
                mattrib c label={" "};
                print c [rowname="Random effects var. component = "];
				print " "; 
				print "Higgins & Thompson 2002 indices of heterogeneity (from moment estimates)";
				print H2 I2 R2;


%MEND meanes;
run; quit;


%MACRO metaf(es,w,x,dsn=_last_,model=fe) ;
*---Revised Lipsey-Wilson macro, 7/2005 (L. Madden). This version CORRECTLY does the ML estimation
	(but maybe not REML). Macro made available for distribution courtesy of David Wilson.
	If you use, please cite book: Practical Meta-Analyis, by M. W. Lipsey & D. B. Wilson. 
	2001. Sage Publications, Thousand Oaks, USA.;
*---Does moment or ML estimation;
      proc iml;

        use &dsn ;
        read all var {&es} into es  where(&es^=. & &w^=. & &x^=.) ;
        read all var {&w}  into w   where(&es^=. & &w^=. & &x^=.) ;
        read all var {&x}  into grp where(&es^=. & &w^=. & &x^=.) ;

        k = nrow(es)  ;
        df = k - 1   ;
        x = design(grp) ;
        p = j(1,ncol(x),1) ;
        j = j(k,1,1) ;
        v = 1/w ;

        %if %upcase(&model) = MM | %upcase(&model) = ML %then
           %do;
                  xw = x#(w*p) ; xwx = T(xw)*x ;
                  B = inv(xwx)*T(xw)*es ;
                  qe = sum(es#w#es) - T(B)*xwx*B ;
                  ww = w#w ; xww = x#(ww*p) ; xwwx = T(xww)*x ;
                  c = (qe-(k-ncol(x)))*inv(sum(w) - trace((xwwx)*inv(xwx))) ;
				  
                  if c<0 then c = 0 ;
                  w = 1/(v + c) ;
            %end ;

        %if %upcase(&model) = ML %then
           %do i=1 %to 100;
                          xw = x#(w*p) ; xwx = T(xw)*x ;
                          B = inv(xwx)*T(xw)*es ;
                          r = es - x*B ;
                          c = sum((w#w)#(r#r-(v))) /
                                (sum(w#w)) ;
                          if c<0 then c = 0 ;
			  w = 1/(v+c) ;
           %end ;

	%if %upcase(&model) = ML %then
			%do ;  se_c = sqrt(2/sum(w#w)) ; %end ;

	  group = char(inv(T(x)*x)*T(x)*grp,8,0) ;
	  means = T(T(T(x#(w*p))*es)*inv(T(x#(w*p))*x)) ;
	  grpns = vecdiag(T(x)*x) ;
	  q =  T(x#(w*p))*(es##2)-T(T((T(x#(w*p))*es)##2)*inv(T(x#(w*p))*x)) ;
	  qt = sum(es##2#w) - sum(es#w)**2/sum(w) ;
	  qw = sum(q) ;
	  qb = qt - qw ;
	  dfb = ncol(x) - 1 ;
	  dfw = nrow(es) - ncol(x) ;
	  dft = nrow(es) - 1 ;
	  pb   = 1 - probchi(qb,dfb) ;
	  pw   = 1 - probchi(qw,dfw) ;
	  ptot = 1 - probchi(qt,dft) ;
	  se = sqrt(vecdiag(inv(T(x#(w*p))*x)));
	  tvalues = means#vecdiag(inv(diag(se))) ;
	  pt = 1-probf(tvalues#tvalues,1,dfw) ;
	  lmeans = means - (se#1.96) ;
	  umeans = means + (se#1.96) ;
	  gmean = sum(es#w)/sum(w) ;
	  segmean = sqrt(1/sum(w)) ;
	  df = grpns - 1 ;    
	  pq = 1 - probchi(q,df) ;

      %if %upcase(&model) = MM %then
        %do ; print "--------- Random Effects Model ------" ;
              print "-- estimated via method of moments --" ;
        print c [label= ' '] [rowname = 'Random eff. var. component = ']
                        [format=12.8];
        %end ;

      %if %upcase(&model) = ML %then
        %do ; print "--------- Random Effects Model ------" ;
              print "-- estimated via iterative full information maximum likelihood --" ;
        mlc = c // se_c ;
		print mlc [label= ' '] [rowname = {'Random eff. var. component  = '
                  'St. Error of var. component = '}]
                  [format=12.8];
        %end ;

	  print  "------- Analog ANOVA table (Homogeneity Q)  -------";
	  qtab1 = qb // qw // qt ;
	  qtab2 = dfb // dfw // dft ;
	  qtab3 = pb // pw // ptot ;
	  print qtab1 [rowname={"Between","Within","Total"}] 
	  			  [colname="Q"]		[label="Source"] [format=12.5]
	  		qtab2 [label="df"] 
	  		qtab3 [label="p"] [format=12.5];
	
	  print "------- Q by Group -------" ;
	  print q [rowname=group] 
	  			  [colname="Q"]	[label="Group"] [format=12.5]
		 	df [label="df"]
			pq [label="p"] [format=12.5];
	  
	  print "------- Effect Size Results by Group -------" ;
	  means = means // gmean ;
	  se = se // segmean ;
	  lmeans = lmeans // gmean-(segmean*1.96) ;
	  umeans = umeans // gmean+(segmean*1.96) ;
	  tvalues = tvalues // gmean/segmean ;
	  pt = pt // 1-probf((gmean/segmean)#(gmean/segmean),1,dfw) ;
	  grpns = grpns // k ;
	  group = group // "   Total" ;
	  print means [rowname=group] [colname="Means"] [label="Group"]
	  		[format=10.5]
			se [label="SE"] [format=10.5]
			lmeans [label='-95%CI'] [format=10.5]
			umeans [label='+95%CI'] [format=10.5]
			tvalues  [label="t"] [format=10.5]
			pt  [label="p"] [format=10.5]
			grpns [label="n"];

%MEND metaf;
run;quit;


%macro moments(dfile=,es=,wgt=,study=);
*---Experimental macro, provided as-is (that is, provided with no warranty).;

*---Macro is an alternative to the 'meanes' macro by Wilson (see above). 
	This uses SAS procedures and data steps to conduct a univariate MOMENT-based 
	meta analysis, instead of an IML program. In some ways, the advantage of this 
	macro is for educational purposes -- demonstrating how to obtain the needed 
	statistics for the simple case of no moderators. Program also is useful 
	for obtaining the I-squared heterogeneity index of Higgins and Thompson.;  
*---Macro written by L. Madden.;

*---In general, I prefer likelihood-based meta-analyses, but many use the
	moment approach (thus, this macro).;

/*
dfile		name of the sas data file with data
es			name of variable in dfile with the effect size response variable
wgt			name of the weight variable in dfile. This needs to be the
			inverse of the "sampling variance" for each study
study		variable name for the labeled study (trial) in dfile

Macro creates various variables and data files starting with the underline symbol (_). 
As a precaution, it is best if you do not have any variables or file names that
start with this symbol. Macro also overwrites titles starting at title2. 

After running the macro, the data files _q_out2 and _q_out3, which contain the 
results, are available for further processing (if needed).
*/

proc glm data=&dfile;
title2 "Cochran's Q, df, test of homogeneity, and Higgins-Thompson H2 & I2";
*ods listing select overallanova;		*<--look only at a table that contains SSE;
ods output overallanova=_q_out;			*<--store Q (=SSE) for manipulation below;
class &study;
weight &wgt;
model &es = / solution;
run;quit;
*---Only one piece of output from PROC GLM is used (the rest is meaningless here).;

data _q_out2; set _q_out(where=(Source='Error'));	*<--manipulate the Q value;
	Q = ss;
	H2 = Q/df;					*<--H-squared statistic of Higgins and Thompson;
	I2 = 100*(H2-1)/H2;			*<--I-squared statistic of Higgins and Thompson;
	if (I2 lt 0) then I2 = 0;
	Pq = 1 - probchi(Q,df);		*<--P value for test of homogeneity (Cochran Q test);
	keep Q  df Pq H2 I2 ;
run;
/*
proc print data=_q_out2;
	var Q df Pq H2 I2 ;
run;
*/

/*
One can get the MOMENT-based estimate of the among-study variance using the
following code, which uses the value of Q calculated in the above file.
*/

*ods listing close;
data _dfile2; set &dfile; 			*<--temporary file;
wgt2=&wgt**2;						*<--get weights squared (same as sampling SD to 4th power);
run;

*proc print data=_dfile2;run;

proc means data=_dfile2 sum noprint;	*<--get and store the sum of weights and weights-squared;
var &wgt wgt2;
output out= _dfile2out  sum(&wgt)=wgts sum(wgt2)=wgt2s;
run;

data _q_out3; merge _q_out2 _dfile2out;	*<--combine/merge files (Q, etc., with sum of weights);
data _q_out3; set _q_out3;				*<--get c coefficient (see Appendix in Madden & Paul);
c = wgts - (wgt2s/wgts);	
variance_MM = (Q - df)/c;				*<--get moment-based among-study variance estimate;
if (variance_MM lt 0) then variance_MM = 0;
run;

data _dtemp; 
if _n_ = 1 then set _q_out3;
set &dfile;
newweight = 1/(variance_MM + (1/&wgt));
joint = newweight*&es;
*proc print data=_dtemp;run;

proc means data=_dtemp noprint;
output out=_dfile2out sum(newweight)=snewweight sum(joint)=sjoint;
data _dfile2out; set _dfile2out;
meanES_MM = sjoint/snewweight;
SE_meanES_MM = snewweight**(-0.50);

data _q_out3; merge _q_out3 _dfile2out;
*ods listing;
run;
title2 'Moment-based estimate of the among-study variance, etc. (cf. macro results)';
proc print data=_q_out3;
var Q c variance_MM meanES_MM SE_meanES_MM;
proc print data=_q_out2;
	var Q df Pq H2 I2 ;
run;
ods results off;
proc datasets nolist ;delete _dfile2 _dfile2out _dtemp _q_out  run;
ods results;
run;
title2;
run;
%mend moments;
run;

%macro funnel(dfile=,es=,wgt=,study=,minprec=2,maxprec=80,numb=25);
*---Experimental macro, provided as-is (that is, provided with no warranty);

*---Macro produces a funnel graph and a radial plot for assessing whether or
	not a random study effect is needed. These diagnostics, especially the
	funnel graph, are very common in some disciplines (but not yet in plant 
	pathology). Macro written by L. Madden.;

*---Funnel graph: plot of "precision" vs. effect size, with a vertical line
	for FIXED-effect common effect size, and curves for the confidence intervals
	around the common effect size. If "too many" points are outside the curves,
	this is evidence for a random effects analysis.
	In this application, precision is the inverse of the within-study SE;
*---Ideally, the funnel graph should be symmetrical. Gaps (say, on the lower
	left or right), can indicate PUBLICATION BIAS, where nonsignificant results
	are not published or made available. However, the gaps could be due to
	other factors, so this is just a guide. These graphs originiated with, or
	were advocated by, Light and Pillemer, Sterne et al., and Egger et al.;

*---Radial plots, also known as Galbraith plots, show the standardized effect
	size (_es_/SE) versus precision (1/SE) for each study. The slope of the line
	through these points is the FIXED-effects common effect size. If many points
	fall outside the confidence bands, there is evidence for a random effects
	analysis. Gaps are indicative of PUBLICATION BIAS.;
	
/*
dfile		name of the sas data file with data
es			name of variable in dfile with the effect size response variable
wgt			name of the weight variable in dfile. This needs to be the
			inverse of the "sampling variance" for each study
study		variable name for the labeled study (trial) in dfile
minprec		Minimum precision value used in the graph (set by trial and error, so
			that the full range of data is encompassed by the curves
maxprec		Maximum precision value used in the graph (set by trial and error, so
			that the full range of data is encompassed by the curves
numb		Number of precision values to make from minprect to maxprec (default25)

Macro creates various variables and data files starting with the underline symbol (_). 
As a precaution, it is best if you do not have any variables or file names that
start with this symbol. Macro also overwrites titles starting at title2. 
One can ignore tabular output here (just use the plots). Graphs are made with 
the new SGPLOT procedure, which requires sas 9.2 or later.

After running the macro, the data files _pred1s and _funnel are available for 
further use, if desired.
*/

title2 'Funnel and radial plots';
*ods listing close;
*ods html;
*ods graphics on  /* / width=6in */ ;

data _ESdata; set &dfile;
keep &es  _precdata _StES;
_precdata = sqrt(&wgt);
_StES = &es*_precdata;
run;
*proc print data=_ESdata;run;

proc mixed data=_ESdata;	*---a fixed-effect (common-effect) analysis.;
title3 '(PROC MIXED used to get fixed-effect values, go right to the plots)';
ods output solutionf=_sf; 
model _StES = _precdata / s noint outp=_pred1;
parms (1) / hold=1;
run;
title3;
data _null_; set _sf;
call symput('ref',estimate);

run;


data _prec; 

_incr = (&maxPrec - &minPrec)/&numb;
do _prec = &minPrec to &maxPrec by _incr;

	_ES1 = &ref + 2*(1/_prec);
	_ES2 = &ref - 2*(1/_prec);
	output;
end;
run;
*proc print data=_prec;run;




data _funnel; merge _prec _ESdata;
run;
*proc print data=_funnel;run;

data _pred1; set _pred1;
low = pred - 2;
high = pred + 2;

proc sort data=_pred1 out=_pred1s;
by _precdata;run;
*proc print data=_pred1s;run;



data _pred1s; set _pred1s;
label _precdata = 1/SE _StES=Standardized Effect Size;
data _funnel; set _funnel;
label &es=Effect size _precdata=1/SE;
run;

data _null_;
put "Radial plot data in file _pred1s";
put "funnel data in file _funnel";
run;

run;


proc sgplot data=_funnel;
series x=_ES1 y=_prec / lineattrs=(color=red pattern=2 thickness=2);
series x=_ES2 y=_prec / lineattrs=(color=red pattern=2 thickness=2);
scatter x=&es y=_precdata / markerattrs=(color=blue symbol=squarefilled size=7pt);
refline &ref / axis=x  lineattrs=(color=blue pattern=1 thickness=2);
*refline 0 / axis=x lineattrs=(color=gray pattern=3 thickness=1);
run;

proc sgplot data=_pred1s;
scatter  x=_precdata y=_StES / markerattrs=(color=red symbol=squarefilled size=7pt);
series x=_precdata y=pred / lineattrs=(color=red pattern=1 thickness=2);
series x=_precdata y=low / lineattrs=(color=blue pattern=2 thickness=1);
series x=_precdata y=high / lineattrs=(color=blue pattern=2 thickness=1);
run;   

*ods graphics off;
*ods html close;
*ods listing;
run;

title2;
ods results off;
proc datasets nolist ; delete _esdata  _prec _pred1  _sf; run; ods results;run;
%mend funnel;
run;

%macro Probnewstudy(meanES=,SIGMA2=,numb=20,minT=,maxT=);
*---Experimental macro, provided as-is (that is, with no warranty).;

*---Determine probability that a randomly selected study has an Effect Size greater (less) than
	the constant _T, based on the meanES and between-study variance from a random-effects
	meta-analysis. Method described in Madden and Paul (Phytopathology 101:16-30 [2011]).
	Key references given in that paper. Macro written by L. Madden. ;
*---Macro overwrites title2 and higher.;

/*
meanES		Mean (expected) Effect Size. Use meanES estimated from a random-effects meta-analysis
SIGMA2		Among-study variance (from a random-effects meta-analysis)
minT		Minimum value of Effect-Size Constant considered 
maxT		Maximum value of Effect-Size Constant considered
numb		Number of constant Effect Sizes between minT and maxT (default is 20)
			(one actually gets one more than the value of numb)

Results are put in the data file _gener, which can be accessed after running the macro,
if desired.
*/

title2 'Probability that Effect Size is greater (or less) than specified constants';
title3 '(Investigator must use context to know which one of two probabilities is needed)';
data _gener; 
_meanES = &meanES;
_SIG2 = &SIGMA2;
label  _1mpz='Pr(ES > const)' _meanES='mean ES' _SIG2='among-study var' 
_pz='Pr(ES < const)' 
_t='const ES';

_SIGMA=sqrt(&SIGMA2);
_incr = (&maxT - &minT)/&numb;

do 	_t =  &minT to &maxT  by _incr;
	_z = (_t - &meanES)/_SIGMA;
	_pz = cdf('normal',_z);
	_1mpz = 1 - _pz;
	output;
label _t='Constant' _z='Z statistic' _pz='Prob(less than)' _1mpz='Prob(greater than)';
end;
put "data file is _gener";
run;
proc print data=_gener label; var _meanES _SIG2 _t _z _pz _1mpz; 
run;
*proc datasets nolist;*run;
%mend Probnewstudy;
run;

%macro biasbound(dfile=,wgt=,SIGMA2=,plotbiasmax=,plotbiasref=,K=,new=);
*---Experimental macro, provided as-is (that is, with no warranty).;

*---Macro calcuates the absolute value of the bias bound from a univariate meta-analysis,
	based on the method developed by Copas and Jackson. Basic concept: find the
	magnitude of the bias (upper limit) for the estimated effect size if there is from 
	1 to NEW additional studies that were not included in the meta-analysis.;
*---Tabular and graphic output is generated.
	Results are stored in data file _gg0, which can be accessed after running the macro.;
*---Macro written by L. Madden.;

/*
dfile		Name of sas data file with effect sizes and related items
wgt			Weight variable (inverse of the "sampling variance") for each
			study. The same weight used in the meta-analysis
SIGMA2		Among-study variance (estimated from a random-effects meta-analysis)
plotbiasmax	Maximum "y value" in the graph of |bias bound| vs. # of unrepored studies.
			Good choice: absoulte value of the mean ES estimated from the random-effects
			meta-analysis.
plotbiasref	A horizontal reference line on the graph. Could be the absolute value of 
			lower 95% confidence limit for mean ES. If you don't want a reference
			line, then give a plotbiasref value greater than plotbiasmax.
K			Number of studies in the actual meta-analysis.
new			Maxinum number of "additional" studies that could exist. The bias
			bound is found for 1-->new addtional studies. One choice for NEW:
			if the original meta-analysis was based on K studies, then also
			use K for NEW. 

			If the bias-bound curve does not cross or come too close to the mean
			ES, then publication bias is likely not too much of a concern.
*/
title2 '|Bias bound| vs. unobserved studies';
data _gg00; set &dfile;
_pr = (1/&wgt);
_pr2 = (_pr + &SIGMA2)**(-0.5);
_pr3 = (_pr + &SIGMA2)**(-1);
run;
*proc print data=_gg00;run;
proc means data=_gg00  noprint ;
var _pr2 _pr3;
output out=_gg000 sum(_pr2)=_pr2 sum(_pr3)=_pr3; 
proc print data=_gg000;run;
data _gg000; set _gg000;
_rat1 = _pr2/_pr3;
call symput('_precR',_rat1);

data _gg0;
do m = 1 to &new;
	ratio = &K/(&K+m);
	middle = quantile('NORMAL',ratio);
	prob=pdf('NORMAL',middle);
	_bias = (1/ratio)*prob*&_precR;
	
	output;

	end;
label _bias='|Bias bound|' m='Additional studies';

proc print data=_gg0 label;
var m _bias;
run;

*---graph the bias bound vs. number of unobserved studies;
*ods html;
*ods graphics on;
proc sgplot data=_gg0;
series x=m y=_bias / 
		lineattrs=(color=blue thickness=3 pattern=1);
refline &plotbiasref / axis=y lineattrs=(color=grey pattern=3 thickness=1);
xaxis  label="Number of unobserved studies" ;
yaxis  label="|Bias bound|" max=&plotbiasmax min=0;
run;
*ods graphics off;
*ods html close;
run;
*proc datasets nolist;*run;
%mend biasbound;


%macro metapower(meanES=,SE=,K=,dfAdjust=1,alpha=0.05,nullmu=0);
title2 'Power analysis for random-effects meta-analysis (1- and 2-sided)';
*---Experimental macro, provided as-is (that is, with no warranty).;
*---Determine the power of the meta-analysis to detect true non-null effects, if one 
	had another set of K studies with the same results, with the assumption that
	the null hypothesis (of no effect) is false (i.e., so that the alternative
	hypothesis is true).;
*---The null hypothesis is 		Ho: ES = nullmu

	Alternative hypothesis is:	Ha: ES ne nullmu 	(not equal)	, or
								Ha: ES >  nullmu				, or
								Ha: ES <  nullmu
	All three Ha hypotheses are evaluated. Note that a large number of decimal places are displayed,
	so that one can compare different scenarios.
;

/*
meanES		Mean (expected) Effect Size. Use meanES estimated from a random-effects meta-analysis
SE			Standard Error of estimated Effect Size (from a random-effects meta-analysis)
K			Number of studies in the meta-analysis
alpha		Pre-determined type I error rate constant (usually 0.05). That is, if one is
			performing a test of Ho: ES = nullmu with the chosen alpha.
nullmu		Constant value of the null hypothesis, that is, Ho: ES = nullmu
dfAdjust	df for the hypothesis test is based on number of studies minus dfAdjust.
			Typically, this is df = K - dfAdjust, and dfAdjust=1. However, some
			argue for more stringent conditions (e.g., dfAdjust=2). 

Results are put in the data file _fullpower, which can be accessed after running the macro,
if desired.
*/
data _fullpower;
	_meanES = &meanES;
	_SE = &SE;
	_mu = &nullmu;
	_df = &K - &dfAdjust;
	_tstatistic = &meanES/&SE;
	_noncen = ((&meanES - &nullmu)/&SE)**2;			*---noncen, power, etc., are for LRdon;
				
	_Fcrit = finv(1-&alpha,1,_df,0);
	_power = 1 - probf(_Fcrit,1,_df,_noncen);

	nc0 = (&meanES - &nullmu) / &SE ;
	tcrit1 = tinv(1-&alpha/1, _df);					*<--one-sided;
	tcrit2 = tinv(&alpha/1, _df);
	
	Pwr10 = 1 - probt(tcrit1,_df,nc0);
	Pwr20 = probt(tcrit2,_df,nc0);

	format _power 16.12 Pwr10 16.12 Pwr20 16.12;
	*label _power='Power';
	label _power='Power(2-side)' _mu='NullMean' Pwr10='Power(upper 1-side)' Pwr20='Power(lower 1-side)'
		_meanES='mean ES' _SE='Std Error' _df='df' nc0='NonCentral param';

run;
proc print data=_fullpower label;  
var _meanES _SE _df _mu nc0 _power Pwr10 Pwr20; run;
data _null_;
put "results are in file _fullpower";
run;
title2;
%mend metapower;


%macro metapower2(meanES=,SE=,K=,dfAdjust=1,alpha=0.05,nullmumin=0,nullmumax=,numb=25);
title2 'Power analysis for random-effects meta-analysis (1- and 2-sided)';

*---Experimental macro, provided as-is (that is, with no warranty).;
*---Determine the power of the meta-analysis to detect true non-null effects, if one 
	had another set of K studies with the same results, with the assumption that
	the null hypothesis (of no effect) is false (i.e., so that the alternative
	hypothesis is true).;
*---The null hypothesis is 		Ho: ES = nullmu

	Alternative hypothesis is:	Ha: ES ne nullmu 	(not equal)	, or
								Ha: ES >  nullmu				, or
								Ha: ES <  nullmu
	All three Ha hypotheses are evaluated. Note that a large number of decimal places are displayed,
	so that one can compare different scenarios.

	A range of values for the expected value under the null hypothesis (nullmu) is considered,
	from nullmumin to nullmumax. In other words, this macro is the same as metapower, except
	that multiple values of nullmu are evaluated.;

*---Results are stored in data file _fullpower2, which can be accessed after running the macro.;

/*
meanES		Mean (expected) Effect Size. Use meanES estimated from a random-effects meta-analysis
SE			Standard Error of estimated Effect Size (from a random-effects meta-analysis)
K			Number of studies in the original meta-analysis
alpha		Pre-determined type I error rate constant (usually 0.05). That is, if one is
			performing a test of Ho: ES = nullmu with the chosen alpha.
nullmumin	Smallest mean ES under Ho (ES = nullmu) considered
nullmumax	Largest mean ES under Ho (ES = nullmu) considered
dfAdjust	df for the hypothesis test is based on number of studies minus dfAdjust.
			Typically, this is df = K - dfAdjust, and dfAdjust=1. However, some
			argue for more stringent conditions (e.g., dfAdjust=2). 
numb		Number of distinct nullmeans evaluated (between nullmumin and nullmax)
			Default of 25 values.

Graph made with the SGPLOT procedure (requires sas 9.2 or later).

*/
data _fullpower2;
	_meanES = &meanES;
	_SE = &SE;
_incr = (&nullmumax - &nullmumin)/&numb;
_df = &K - &dfAdjust;
_K = &K;

do _mu = &nullmumin to &nullmumax by _incr;
	_tstatistic = &meanES/&SE;
	_noncen = ((&meanES - _mu)/&SE)**2;					*---noncen, power, etc., are for LRdon;
				
	_Fcrit = finv(1-&alpha,1,_df,0);
	_power = 1 - probf(_Fcrit,1,_df,_noncen);

	nc0 = (&meanES - _mu) / &SE ;
	tcrit1 = tinv(1-&alpha/1, _df);					*<--one-sided;
	tcrit2 = tinv(&alpha/1, _df);
	
	Pwr10 = 1 - probt(tcrit1,_df,nc0);
	Pwr20 = probt(tcrit2,_df,nc0);
	output;
end;

format _power 16.12 Pwr10 16.12 Pwr20 16.12;
label _power='Power(2-side)' _mu='NullMean' Pwr10='Power(upper 1-side)' 
Pwr20='Power(lower 1-side)' _df='df' _K='No of studies'
_meanES='mean ES' _SE='Std Error' _df='df' nc0='NonCentral param'  
;
run;

*ods html;
*ods graphics on / width=6in;
proc print data=_fullpower2 label; 
var _meanES _SE _df  _K _mu nc0 _power Pwr10 Pwr20; run;


proc sgplot data=_fullpower2;
series x=_mu y=_power / lineattrs=(color=blue thickness=3 pattern=1);;
series x=_mu y=Pwr10 / lineattrs=(color=blue thickness=2 pattern=3);;
series x=_mu y=Pwr20 / lineattrs=(color=blue thickness=2 pattern=4);;
yaxis label="Power"; xaxis label="Null hypothesis mean";
run;
*ods graphics ;
*ods html close;
run;
data _null_;
put "results in file _fullpower2";
title2;
run;
%mend metapower2;
run;


%macro metapower3(meanES=,SE=,alpha=0.05,nullmu=0,Korig=,Kmin=2,Kmax=);
title2 'Power analysis for random-effects meta-analysis (1- and 2-sided)';
*title3 '(range of number of studies for identified ES under the null hypothesis)';

*---Experimental macro, provided as-is (that is, with no warranty).;
*---Determine the power of the meta-analysis to detect true non-null effects, if one 
	had another set of K studies with the same results, with the assumption that
	the null hypothesis (of no effect) is false (i.e., so that the alternative
	hypothesis is true).;
*---The null hypothesis is 		Ho: ES = nullmu

	Alternative hypothesis is:	Ha: ES ne nullmu	 (not equal), or
								Ha: ES >  nullmu				, or
								Ha: ES <  nullmu
	All three Ha s are evaluated. Note that a large number of decimal places are displayed,
	so that one can compare different scenarios.

	A range of number of studies (K) is considered (from Kmin to Kmax), all for one 
	null-hypothesis mean (nullmu). Separate results for K: Kmin-->Kmax. This macro is the 
	same as %metapower, except that a range of K values are considered.;
*---The macro is based on the "theoretical power" calculations shown in Madden
	and Paul (2011). A more accurate approach is based on simulation (also described in
	Madden and Paul), but this macro does not do the latter.;

*---Macro also calculates the 95% confidence interval for the expected effect size for
	all K values between Kmin and Kmax.;

*---Results are stored in data file _fullpower3, which can be accessed after running the macro.;

/*
meanES		Mean (expected) Effect Size. Use meanES estimated from a random-effects meta-analysis
SE			Standard Error of estimated Effect Size (from a random-effects meta-analysis)
Korig		Original number of studies (K) in the meta-analysis that results in meanES and SE
Kmin		Minimum number of studies in the original meta-analysis
Kmax		Maximun number of studies in the meta-analysis
alpha		Pre-determined type I error rate constant (usually 0.05). That is, if one is
			performing a test of Ho: ES = nullmu with the chosen alpha.
nullmu		Constant value of the null hypothesis, that is, Ho: ES = nullmu
dfAdjust	df for the hypothesis test is based on number of studies minus dfAdjust.
			Typically, this is df = K - dfAdjust, and dfAdjust=1. However, some
			argue for more stringent conditions (e.g., dfAdjust=2). 

Graph made with SGPLOT procedure (requires sas 9.2 or later).
*/

data _fullpower3;

_mnES = &meanES;
_C = &SE/(&Korig**(-0.5));	*<--a Standard-Deviation-type index;
nullmean=&nullmu;
do _K = &Kmin to &Kmax ;
	
	_SE = _C*(_K**(-0.5));
	_tstatistic = &meanES/_SE;
	_noncen = ((&meanES - &nullmu)/_SE)**2;					*---noncen, power, etc.;
				
	_Fcrit = finv(1-&alpha,1,_K-1,0);
	_power = 1 - probf(_Fcrit,1,_K-1,_noncen);

	nc0 = (&meanES - &nullmu) / _SE ;
	tcrit1 = tinv(1-&alpha/1, _K-1);						*<--one-sided;
	tcrit2 = tinv(&alpha/1, _K-1);
	
	Pwr10 = 1 - probt(tcrit1,_K-1,nc0);
	Pwr20 = probt(tcrit2,_K-1,nc0);
	output;
end;

format _power 16.12 Pwr10 16.12 Pwr20 16.12 nc0 7.3 _K 4.0;

label _power='Power(2-side)'  Pwr10='Power(upper 1-side)' Pwr20='Power(lower 1-side)' 
nc0='Noncentral Parm' _K='Numb of studies' _df='df' _mnES='Mean ES' ;

data _fullpower3; set _fullpower3;
_upperl = &meanES + sqrt(_Fcrit)*_SE;
_lowerl = &meanES - sqrt(_Fcrit)*_SE;
label _upperl="Upper limit" _lowerl="Lower limit";
run;

*ods html;
*ods graphics on;
proc print data=_fullpower3 label; var _mnES _K  nc0  nullmean _power Pwr10 
		Pwr20 ; 
proc print data=_fullpower3 label; var _mnES _K _lowerl _upperl;
run;

proc sgplot data=_fullpower3;
series x=_K y=_power / lineattrs=(color=blue thickness=3 pattern=1);;
series x=_K y=Pwr10 / lineattrs=(color=blue thickness=2 pattern=3);;
series x=_K y=Pwr20 / lineattrs=(color=blue thickness=2 pattern=4);;
yaxis label="Power"; xaxis label="Number of studies";
run;

proc sgplot data=_fullpower3;
band x=_K upper=_upperl lower=_lowerl / transparency=0.5 fill 
		outline lineattrs=(pattern=solid);
series x=_K y=_mnES / lineattrs=(color=blue thickness=3);;
xaxis type=Log /*logstyle=LogExpand minor*/ label="Number of studies";
yaxis label="Mean and Conf. Int.";
run;
*ods graphics off;
*ods html close;
run;
data _null_;
put "results in file _fullpower3";
title2;
run;
%mend metapower3;
run;



/*
Example from Shah and Dillard (Plant Disease 90: 1413-1418 [2006]). Effect size in 
each study is the slope of the relationship between sweet-corn yield and rust severity. 
This data set corresponds to the processing sweet-corn data in the paper
(see their Fig. 1).

Data set contains the estimated slope and standard error (from a regression analysis in
each study), study identification (study), and several moderator variables. The 95% 
confidence interval for the slope in each study is also given (lowerlimit --> upperlimit).

For a meta-analysis, one needs to define the so-called sampling variance for each study 
and its inverse (the weight). When dealing with regression parameters, the sampling variance
is the square of the standard error of the estimated parameter. The weight is simply
the inverse of the sampling variance.
*/

title 'Yield loss in sweet corn from common rust (Shah & Dillard)';
data shah;
input study $ State $ Variety $ Year slope SE lowerlimit upperlimit MinD MaxD MeanD;	
sampvar = SE**2;			*<--sampling variance for each slope;
wgt = 1/sampvar;			*<--weight used in the meta-analysis;
id = _n_;					*<--number of the studies (in order);
datalines;
1	NY	Zenith	1999	2.13607	2.18587	-3.03268	7.30483	0.00	1.46	0.39
2	NY	Zenith	1998	0.47288	0.18572	0.03372	0.91204	0.01	19.04	3.05
8	NY	Squeen	2000	1.17716	0.14263	0.83989	1.51443	1.50	24.30	6.58
11	NY	Bold	2001	0.33829	0.05394	0.21626	0.46031	20.49	77.31	45.91
22a	NY	Zenith	1997	0.78559	0.24789	-0.00331	1.57450	1.40	19.40	8.10
22b	NY	Rival	1997	-0.50102	0.42602	-1.85679	0.85476	1.50	6.30	3.73
31	MI	HMX83865	1993	0.31888	0.01484	0.27163	0.36612	5.40	57.50	18.73
35	MI	YBelle	1992	0.14559	0.12224	-0.15352	0.44470	2.60	85.80	22.96
50	NY	Jubilee	1984	-0.79923	0.31152	-1.79063	0.19217	8.08	15.50	11.73
59	IL	FSSweet	1984	0.63267	0.02300	0.58577	0.67958	5.59	62.83	38.07
60	IL	FSSweet	1985	1.08843	0.06339	0.95246	1.22439	10.45	55.20	26.51
61	IL	FSSweet	1986	0.66299	0.25871	0.09357	1.23241	1.55	17.65	9.54
62	IL	Gcup	1984	0.85302	0.04194	0.76626	0.93979	15.45	59.91	39.79
63	IL	Gcup	1985	0.55555	0.03612	0.48142	0.62967	7.60	50.08	27.63
64	IL	Gcup	1986	0.15786	0.28335	-0.44282	0.75854	1.67	20.57	6.22
65	IL	Stylepak	1984	0.62416	0.01996	0.58344	0.66488	23.63	67.68	38.94
66	IL	Stylepak	1985	0.37280	0.02808	0.31402	0.43157	7.50	49.84	25.70
67	IL	Stylepak	1986	0.78146	0.14652	0.47233	1.09058	1.42	17.40	6.72
70	IL	SnowWhite	2001	0.59867	0.01999	0.55697	0.64036	23.00	65.00	36.95
71	IL	Sterling	2001	0.40333	0.01810	0.36559	0.44108	18.00	59.00	35.81
;
run;

*---Look at the data;
proc print data=shah;
var study slope SE sampvar wgt state variety year meanD;
run;


/*
Now create Forest plots of the effect sizes (ES). The new
SGPLOT procedure is used here (works for sas 9.2 and later). 

SOME IMPORTANT TERMS TO BE USED IN SGPLOT PROCEDURE:
x:			the Effect Size (ES) of the study
y:			the study label (ID)
xerrorlower:lower limit for interval in plot 
			(can be ES - standard error, or lower confidence limit for ES)
xerrorupper:upper limit for interval in plot
			(can be ES + standard error, or upper confidence limit for ES)
refline:	value that indicates "no effect" (often 0)
xaxis label:label of the effect size in the graph
yaxis label:label of the study ID in the graph
xaxis max:	upper limit in graph for the ES (optional)
xaxis min:	lower limit in graph for the ES (optional)

*/

*---In SAS 9.3, the html output destination is automatically turned on;

proc sgplot data=shah;
title2 'Forest plot';
scatter x=slope y=study / xerrorlower=lowerlimit xerrorupper=upperlimit
		markerattrs=(color=blue symbol=squarefilled size=9pt)
		errorbarattrs=(color=blue pattern=1 thickness=1);
refline 0 / axis=x lineattrs=(color=black pattern=1 thickness=1.5); *<--reference line (null hypothesis);
xaxis label="Slope for yield loss" ;
yaxis type=discrete discreteorder=data label="Study" ;
run;

*---Now control the lower and upper limits because of the wide intervals above;
proc sgplot data=shah;	
title2 'Forest plot';	
scatter x=slope y=study / xerrorlower=lowerlimit xerrorupper=upperlimit
		markerattrs=(color=blue symbol=squarefilled size=9pt)
		errorbarattrs=(color=blue pattern=1 thickness=1);
refline 0 / axis=x lineattrs=(color=black pattern=1 thickness=1.5); *<--reference line (null hypothesis);
xaxis max= 4 min=-2 label="Slope for yield loss";		*<--cut off the extemes of CIs for one study;
yaxis type=discrete discreteorder=data label="Study";
run;

*---The following vertical-bar version of the Forest plot can only be done in 9.3 (and later);
proc sgplot data=shah;
title2 'Forest plot';	
vbarparm category=study response=slope / limitupper=upperlimit limitlower=lowerlimit limitattrs=(thickness=.0001in);
xaxis type=discrete fitpolicy=thin;
yaxis max=4 min=-2;
run;



/*	Get a funnel plot and a radial (Galbraith) plot. Helps to identify publication
	bias (gaps in the graphs), and also the need to use random-effects analysis
	(when there are multiple data points outside the lines/curves. 
	In the Results window (for html), you generally should just scroll past all
	the table output, and just look at the graphs. 

dfile		name of the sas data file with data
es			name of variable in dfile with the effect size response variable
wgt			name of the weight variable in dfile. This needs to be the
			inverse of the "sampling variance" for each study
study		variable name for the labeled study (trial) in dfile
minprec		Minimum precision value used in the graph (set by trial and error, so
			that the full range of data is encompassed by the curves
maxprec		Maximum precision value used in the graph (set by trial and error, so
			that the full range of data is encompassed by the curves
numb		Number of precision values to make from minprect to maxprec (default25)
*/

%funnel(dfile=shah,es=slope,wgt=wgt,study=study,minprec=2,maxprec=70);run;



*---Conduct a meta-analysis using the method of moments (using macro by Lipsey and Wilson).;
%meanes(slope,wgt,dsn=shah,print=raw) ;


*---Now do a random-effects meta-analysis based on likelihood principles. We find this a 
	better approach, although moment-based analyses are extremely common (based on
	history).;
*---One can either do a Maximum Likelihood (ML) or Restricted Maximum Likelihood (REML)
	analysis. The former is best suited for large numbers of studies (say, >30), although
	some authors like to use ML for all meta-analyses. With 20 studies in the example, 
	REML is probabably better, but we show both here (for completion and comparison). ;

/*	Model is: z_i = zeta + u_i + e_i
	Where 	z_i is the estimated effect size for the i-th study,
			zeta is the true expected effect size across all studies
				(estimate is labeled INTERCEPT in the MIXED output),
			u_i is the random effect of the i-study on the effect size
				(between-study variability term),
			e_i is the within-study random effect or residual
				(sampling variability term, assumed known for each study)
*/


proc mixed data=shah covtest method=REML;
title2 'random effects, REML';
class study;							*<--Make study (trial) a class variable;
weight wgt;								*<--Specify 1/samp.var as the weight (to get fixed e_i);
model slope = / chisq s ddfm=betwithin;	*<--No predictors in the model. Options for getting a  
											chi-square test and for the denominator df method.;
random study;							*<--Indicate that STUDY is a random effect (u_i);
repeated;								*<--Placeholder. Reminder that there is a residual.;
parms (1) (1) / hold=2;					*<--Initial guesses of the between-study variance and the
											residual, the latter being held at a fixed 1. Required.
											Combination of WEIGHT and fixed residual var (=1) is key.;
estimate 'meanES' int 1 / cl;			*<--Get estimated expected ES (zeta^) and confidence interval;
run;

proc mixed data=shah covtest method=ML;	*<--see REML example for more comments.;
title2 'Random Effects, *ML*';			*<--(see above comments for more details on MIXED);
class study;
weight wgt;
model slope = / chisq s ddf=1000 ;		*<--Make denominator df large to get "large sample" 
											results;
random study;
repeated;
parms (1) (1) / hold=2;
estimate 'meanES' int 1 / cl df=1000;	*<--Make df large for "large sample" results;
run;


*---The diagnostic plots above (Funnel and radial) show that a fixed-effects analysis is not appropriate. 
	This is confirmed by the I2 and R2 indices determined from the MOMENTS-based meta-analysis.
	For completion, however, we show how to do a FIXED-effects analysis using MIXED.;

proc mixed data=shah covtest method=reml;	*<--See earlier runs of MIXED for detailed comments;
title2 'fixed effects analysis';
class study;							
weight wgt;
model slope = / chisq s ddfm=betwithin;		
repeated;
parms (1) / hold=1;						*<--Only one variance parameter (for residual) (held at 1);
estimate 'meanES' int 1 / cl;
run;

/*
One can get the Higgins-Thompson R2 index by dividing the SE from the random effects analysis by the SE from 
the fixed-effects analysis, and squaring them. Here, R2 = (.08560/.007465)**2 = 131 (this is extremely
large).

The confidence interval is automatically calculated with the CL option in MIXED. A _PREDICTION_ INTERVAL 
must be done "by hand". One calculates 
	ES_mean +/- (t*).sqrt(VAR*), 
where
	t*  is critical value from Student t distribution,
	VAR* = SE**2 + (STUDY VAR).

	VAR* = 0.0856**2   + 0.1181 = 0.1254

95% prediction interval: 		0.516 +/- (2).sqrt(.1254):  -0.192 --> 1.223

Note: the confidence interval is							0.337  --> 0.696
*/



*---Often, one likes to include the (random- and/or fixed-effects) overall expected effect size (across
	all studies) in the Forest plot. In SAS, one would do this by first putting the mean ES and the confidence
	limits in a data file, and then stacking the original data file on top of the new one. Also, one
	can also show the PREDICTION INTERVAL for the mean ES, using the same approach.;

data shahmeans;									*<--Can use any label for these extra "studies".;
input studym $ slope lowerlimit upperlimit;		*<--Note that the study ID is given a different name from original file;
datalines;
RAND.-CI .516 .337 .696
FIXED    .493 .477 .508
;
run;
data shahmeans2;								*<--Now augment with the Prediction Intervals;
input studym2 $ slope lowerlimit upperlimit;	*<--Yet, another study ID label;
datalines;
RAND.-PI .516 -.192 1.223
;
run;

*---Stack the data files, and look at the combined file.;
*---ORDER: file with pred. int., file with conf.int., and original file.;

data shah_c; set shahmeans2 shahmeans shah ;
run;
proc print data=shah_c;		*<--this is flipped to get the right order in forest plot;
title2 'Original file augmented with overall fixed- and random-efffects effect sizes';
var study studym studym2 slope lowerlimit upperlimit;
run;

/*
See above comments for the use of SGPLOT procedure to obtain Forest plot.
Here, extra SCATTER statements are used to get the overall expected 
effect size (random or fixed effects), and also the expected effect
size with PREDICTION interval (second procedure, below).
*/

*ods html /* image_dpi=100 */ ;
*ods graphics on /* / height=6in */;
proc sgplot data=shah_c noautolegend;
title2 'Forest Plot';
scatter x=slope y=studym / xerrorlower=lowerlimit xerrorupper=upperlimit
					markerattrs=(color=red symbol=diamondfilled size=11pt)
					errorbarattrs=(color=red pattern=1 thickness=1);
scatter x=slope y=study / xerrorlower=lowerlimit xerrorupper=upperlimit
					markerattrs=(color=blue symbol=squarefilled size=9pt)
					errorbarattrs=(color=blue pattern=1 thickness=1);
refline 0 / axis=x lineattrs=(color=black pattern=1 thickness=1.5);
xaxis label="slope for yield loss" min=-2 max=4;
yaxis type=discrete discreteorder=data label="Study";
run;
proc sgplot data=shah_c noautolegend;
title2 'Forest Plot';
scatter x=slope y=studym2 / xerrorlower=lowerlimit xerrorupper=upperlimit
					markerattrs=(color=black symbol=diamond size=12pt)
					errorbarattrs=(color=black pattern=1 thickness=1);
scatter x=slope y=studym / xerrorlower=lowerlimit xerrorupper=upperlimit
					markerattrs=(color=red symbol=diamondfilled size=11pt)
					errorbarattrs=(color=red pattern=1 thickness=1);
scatter x=slope y=study / xerrorlower=lowerlimit xerrorupper=upperlimit
					markerattrs=(color=blue symbol=squarefilled size=9pt)
					errorbarattrs=(color=blue pattern=1 thickness=1);
refline 0 / axis=x lineattrs=(color=black pattern=1 thickness=1.5);
xaxis label="slope for yield loss" min=-2 max=4;
yaxis type=discrete discreteorder=data label="Study" ;
run;
*ods graphics off;
*ods html close;
run;


/*
Now that we have the results for the random-effects meta-analysis 
(meanES = 0.515, SE = 0.0856, between-study variance [SIGMA2] = 0.1181),
we can perform several applications. First, we estimate the probability
that a new randomly selected study has an effect size larger (or less)
than a pre-specified constant (c). For instance, an ES of 0 may be 
especially of interest (say, 0 could mean that a fungicide had _no_ effect
on disease or yield), but other constants could be equally of interest.

The %Probnewstudy macro does all the calculations, using the method 
described in Madden and Paul (2011). One supplies the results from the prior
(original) analysis, and gives a range of constants of interest 
(minT --> maxT). One gets a printed table of results (in sas file _gener, which
can be accessed later).

One specifies meanES, SIGMA2, minT, maxT, and optionally numb (the number of
constants after minT (one actually gets numb+1 results)). Default numb is 25.
*/

%Probnewstudy(meanES=.515,minT=0,maxT=.7,SIGMA2=.1181);

%Probnewstudy(meanES=.515,minT=.0,maxT=.5,SIGMA2=.1181,numb=2);


/*
Statistical power is one of the great advantages of meta-analysis. In many cases,
the power to detect the true alternative hypothesis (when it is true) can be 
quite high, even when the power for each individual study is (very) low. 
The following macros calculate the power based on the results from a previous 
meta-analysis. 

%metapower simply calculates (estimates) this power, given the mean ES (meanES), 
SE, and number of studies (K) in the previously conducted meta-analysis (at a given 
alpha [typicaly 0.05]). One needs to specifiy the expected value under the null 
hypothesis that one is considering (i.e., nullmu). 

The null hypothesis is 		Ho: ES = nullmu

Alternative hypothesis is:	Ha: ES ne nullmu   (not equal),   or
								Ha: ES >  nullmu			, or
								Ha: ES <  nullmu
All three Ha hypotheses are evaluated. Note that a large number of decimal places 
are displayed, so that one can compare different scenarios.

Results are stored in the sas file _fullpower (which is available for later
access). But when you run the macro another time, the original stored values
are over-written (unless you copied the file).
*/

%metapower(meanES=.515,SE=.0856,K=20,alpha=0.05,nullmu=0);

%metapower(meanES=.515,SE=.0856,K=20,alpha=0.05,nullmu=.25);

%metapower(meanES=.515,SE=.0856,K=20,alpha=0.05,nullmu=0.4);



/*
%metapower2 expands on %metapower by calculating power for a range of nullmu 
values (from nuumumin to nullmumax). Results are also plotted. 

By default, one obtains 26 power values between nullmumin and nullmumax.

One can specify numb to obtain numb+1 power values.
Results are stored in the sas file _fullpower2 (which is availble for later 
access). But when you run the macro another time, the original stored values
are gone.
*/

%metapower2(meanES=.515,SE=.0856,K=20,alpha=0.05,nullmumin=.0,nullmumax=.75);

%metapower2(meanES=.515,SE=.0856,K=20,alpha=0.05,nullmumin=.0,nullmumax=.75,numb=15);


/*
Now, determine power (theoretically) for a range of number of studies, from K = Kmin to Kmax. 
Specify the mean under the null hypothesis (nullmu), and the number of studies
in the original meta-analysis (Korig).
Results are stored in the sas file _fullpower3 (which is availble for later 
access). But when you run the macro another time, the original stored values
are gone.

As a bonus, %metapower3 also determines the confidence interval for the expected
effect size for each value of K. 
*/

%metapower3(meanES=.515,SE=.0856,alpha=0.05,nullmu=.25,Kmin=2,Kmax=25,Korig=20);


/*
Publication bias is a recurring theme in meta-analysis: the selective reporting of 
certain studies rather than all studies that can inform a topic. Because of publication
bias, calculated mean effect sizes (based on the available studies, not the total
of all studies) could be too large in absolute value (the _estimated_ mean will be 
farther from 0 than the _true_ expected effect size. Many approaches have
been developed to assess the possible magnitude of this bias, and some methods try to
correct for this bias. The funnel and radial plots (shown above) can help in this 
assessment. We especially like the method developed by Copas and Jackson (2004). 
Here one considers how biased the _mean_ ES can be if there are m _more_ (new)studies 
that have not been included in the meta-analysis. The method determines the absolute
value of the UPPER BOUND for this bias -- actual bias will almost always be less than
this upper bound(|bound|), but the actual bias is determined by too many unknown
factors to predict. 

The macro determines this |bound| based on the results from a meta-analysis for a 
range of _additional_ studies not used in the (original) meta-analysis.

One specifies the SAS file with the data, indicating the weight (wgt) used
in the meta-analysis, together with the estimated between-study variance (SIGMA2), and
number of studies in this original analysis(K). Then, one specifies the maximum number
of additional studies that could exist (new, a hypothetical value). That is, the
number of new studies not in the original are m = 1, 2, ..., new.

The graphic output is the most useful output of the macro. One needs to indicate
the maximum in the graph of |bound| versus m. (That is, one must give the
maximum value on the y axis. It is recommended that one chooses
the mean ES (actually, its absolute value) from the prior meta-analysis as the maximum
in the graph. The idea is that bias is important if |bound| determined with the 
Copas-Jackson algorithm gets too close to this maximum (or even supasses this maximum). 

One also specifies another reference line. This can be the "lower" limit of a confidence 
interval for the mean ES (from the original meta-analysis).

Results are stored in sas file _gg0 (which is available for later access).
*/

%biasbound(dfile=shah,wgt=wgt,SIGMA2=.1181,K=20,new=20,plotbiasmax=.52,plotbiasref=.34);


/*
All the  material in this file, so far, can be considered frequentist analysis, which is 
the most common paradign for data analysis. However, Bayesian analysis is becomning 
more-and-more popular for many kinds of analyses, including meta-analysis. 
Here we show how to use the MCMC procedure for a Bayesian analysis.

This approach is based on defining a prior probability distribution for each parameter.
It is common to use a so-called noninformative prior. Then, one combines this prior
with the likelihood function for the data to determine the posterior distribution 
for the parameters. Summary measures (e.g., mean, median, quantiles) of the
posterior distributions are used for interpretation. This method is very 
computationally intensive, but now software makes this quite feasible. 

Not teaching the method here.

Note: this version of MCMC works for SAS 9.2 and later. There is a better way
with SAS 9.3, where an explicit random statement can be used.
*/

/*
*ods html;
*ods graphics on;
data shahc; set shah; trial=_n_;	*<--need a numeric code for the studies (not just characters);
proc mcmc data=shahc outpost=postout thin=10 nmc=100000  seed=123
   propcov=quanew  diag=ess;					*<--This approach works for SAS 9.2 and above.;
   title2 'Bayesian meta-analysis, with non-informative priors';
   array zetai[20];								*<--need to specify an array of 20 for the 20 studies;
   parms zeta0 .6 zetai: 0;
   parms sigma2 1 ;

   prior zeta0 ~ normal(0,var=1000);			*<--this the the overall expected ES (with a noninformative prior);
   prior zetai: ~ normal(zeta0,var=sigma2);		*<--the ES for each study, random term (prior based on mean of zeta0
   													and variance of sigma2);

   prior  sigma2 ~ igamma(0.001,scale=0.001);	*<--this is the among-study variance (with a noninformative prior);

   model slope ~ normal(zetai[trial], var=sampvar);	*<--the sampling varinace for each study (1/wgt) is fixed;
  run;
*ods graphics off;
*ods html close;
run;
*/
run;

/*
MODERATOR VARIABLES. 
It is often of major importance to determine if characteristics of the studies affect
the expected effect size. A study characteristic could be the cultivar being 
grown or the level of disease resistance of cultivars being grown (say, L or H).
Another characteristic could be an environmental summary variable for the 
study (total rainfall in a field study, etc.). Study properties or characteristics
are known in meta-analysis as moderator variables. Analysis is based on 
an expansion of the basic linear model, with terms for categorical (factor, class)
moderator variables or continuous moderator variables. 

Analysis is demonstrated below for two moderators in the Shah data set. 
*/


proc mixed data=shah covtest method=reml;
title2 'Moderator variable analysis';
title3 'random effects, effect of mean disease per study [meanD]';
class study ;
weight wgt;
model slope = meanD / chisq s ddfm=betwithin;
random study;
repeated;
parms (1) (1) / hold=2;
run;

*---Use of Store and PROC PLM work only in 9.22 and later versions (not 9.2);
proc mixed data=shah covtest method=reml;
title2 'Moderator variable analysis';
title3 'random effects, variety effect (demonstration)';
class study variety;
weight wgt;
model slope = variety / chisq s ddfm=betwithin;
random study;
repeated;
parms (1) (1) / hold=2;
estimate 'meanES' int 1 / cl;
lsmeans variety / cl;
store sasuser.shahVar;
run;
proc plm source=sasuser.shahVar;
lsmeans variety / lines adjust=simulate stepdown alpha=.05;
run;

